@@ -5,19 +5,23 @@
 
     Changelog builder.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import codecs
 from os import path
-from cgi import escape
+
+from six import iteritems
 
 from sphinx import package_dir
-from sphinx.util import ensuredir, os_path, copy_static_entry
+from sphinx.util import copy_static_entry
+from sphinx.locale import _
 from sphinx.theming import Theme
 from sphinx.builders import Builder
+from sphinx.util.osutil import ensuredir, os_path
 from sphinx.util.console import bold
+from sphinx.util.pycompat import htmlescape
 
 
 class ChangesBuilder(Builder):
@@ -28,7 +32,8 @@ class ChangesBuilder(Builder):
 
     def init(self):
         self.create_template_bridge()
-        Theme.init_themes(self)
+        Theme.init_themes(self.confdir, self.config.html_theme_path,
+                          warn=self.warn)
         self.theme = Theme('default')
         self.templates.init(self, self.theme)
 
@@ -90,9 +95,10 @@ class ChangesBuilder(Builder):
             'version': version,
             'docstitle': self.config.html_title,
             'shorttitle': self.config.html_short_title,
-            'libchanges': sorted(libchanges.iteritems()),
+            'libchanges': sorted(iteritems(libchanges)),
             'apichanges': sorted(apichanges),
-            'otherchanges': sorted(otherchanges.iteritems()),
+            'otherchanges': sorted(iteritems(otherchanges)),
+            'show_copyright': self.config.html_show_copyright,
             'show_sphinx': self.config.html_show_sphinx,
         }
         f = codecs.open(path.join(self.outdir, 'index.html'), 'w', 'utf8')
@@ -111,7 +117,7 @@ class ChangesBuilder(Builder):
                   '.. deprecated:: %s' % version]
 
         def hl(no, line):
-            line = '<a name="L%s"> </a>' % no + escape(line)
+            line = '<a name="L%s"> </a>' % no + htmlescape(line)
             for x in hltext:
                 if x in line:
                     line = '<span class="hl">%s</span>' % line
@@ -120,11 +126,18 @@ class ChangesBuilder(Builder):
 
         self.info(bold('copying source files...'))
         for docname in self.env.all_docs:
-            f = codecs.open(self.env.doc2path(docname), 'r', 'latin1')
-            lines = f.readlines()
+            f = codecs.open(self.env.doc2path(docname), 'r',
+                            self.env.config.source_encoding)
+            try:
+                lines = f.readlines()
+            except UnicodeDecodeError:
+                self.warn('could not read %r for changelog creation' % docname)
+                continue
+            finally:
+                f.close()
             targetfn = path.join(self.outdir, 'rst', os_path(docname)) + '.html'
             ensuredir(path.dirname(targetfn))
-            f = codecs.open(targetfn, 'w', 'latin1')
+            f = codecs.open(targetfn, 'w', 'utf-8')
             try:
                 text = ''.join(hl(i+1, line) for (i, line) in enumerate(lines))
                 ctx = {
@@ -135,17 +148,16 @@ class ChangesBuilder(Builder):
             finally:
                 f.close()
         themectx = dict(('theme_' + key, val) for (key, val) in
-                        self.theme.get_options({}).iteritems())
+                        iteritems(self.theme.get_options({})))
         copy_static_entry(path.join(package_dir, 'themes', 'default',
                                     'static', 'default.css_t'),
-                          path.join(self.outdir, 'default.css_t'),
-                          self, themectx)
+                          self.outdir, self, themectx)
         copy_static_entry(path.join(package_dir, 'themes', 'basic',
                                     'static', 'basic.css'),
-                          path.join(self.outdir, 'basic.css'), self)
+                          self.outdir, self)
 
     def hl(self, text, version):
-        text = escape(text)
+        text = htmlescape(text)
         for directive in ['versionchanged', 'versionadded', 'deprecated']:
             text = text.replace('.. %s:: %s' % (directive, version),
                                 '<b>.. %s:: %s</b>' % (directive, version))
