@@ -48,6 +48,7 @@ class RedirectHandler(HTTPRedirectHandler):
         req.redirect_code = code
         return new_req
 
+
 # create an opener that will simulate a browser user-agent
 opener = build_opener(RedirectHandler)
 opener.addheaders = [('User-agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) '
@@ -56,6 +57,11 @@ opener.addheaders = [('User-agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) '
 
 class HeadRequest(Request):
     """Subclass of urllib2.Request that sends a HEAD request."""
+    def __init__(self, *args, **kwargs):
+        Request.__init__(self, *args, **kwargs)
+        # we do not parse the response in HEAD, so accepting anything is okay
+        self.headers['Accept-encoding'] = '*'
+
     def get_method(self):
         return 'HEAD'
 
@@ -75,11 +81,11 @@ class AnchorCheckParser(HTMLParser):
                 self.found = True
 
 
-def check_anchor(f, hash):
-    """Reads HTML data from a filelike object 'f' searching for anchor 'hash'.
+def check_anchor(f, anchor):
+    """Reads HTML data from a filelike object 'f' searching for *anchor*.
     Returns True if anchor was found, False otherwise.
     """
-    parser = AnchorCheckParser(hash)
+    parser = AnchorCheckParser(anchor)
     try:
         # Read file in chunks of 8192 bytes. If we find a matching anchor, we
         # break the loop early in hopes not to have to download the whole thing.
@@ -137,12 +143,76 @@ class CheckExternalLinksBuilder(Builder):
         if self.app.config.linkcheck_timeout:
             kwargs['timeout'] = self.app.config.linkcheck_timeout
 
+        def check_uri():
+            # split off anchor
+            if '#' in uri:
+                req_url, anchor = uri.split('#', 1)
+            else:
+                req_url = uri
+                anchor = None
+
+            # handle non-ASCII URIs
+            try:
+                req_url.encode('ascii')
+            except UnicodeError:
+                req_url = encode_uri(req_url)
+
+            try:
+                if anchor and self.app.config.linkcheck_anchors and \
+                   not anchor.startswith('!'):
+                    # Read the whole document and see if #anchor exists
+                    # (Anchors starting with ! are ignored since they are
+                    # commonly used for dynamic pages)
+                    req = Request(req_url)
+                    f = opener.open(req, **kwargs)
+                    encoding = 'utf-8'
+                    if hasattr(f.headers, 'get_content_charset'):
+                        encoding = f.headers.get_content_charset() or encoding
+                    else:
+                        encoding = get_content_charset(f) or encoding
+                    found = check_anchor(TextIOWrapper(f, encoding),
+                                         unquote(anchor))
+                    f.close()
+
+                    if not found:
+                        raise Exception("Anchor '%s' not found" % anchor)
+                else:
+                    try:
+                        # try a HEAD request, which should be easier on
+                        # the server and the network
+                        req = HeadRequest(req_url)
+                        f = opener.open(req, **kwargs)
+                        f.close()
+                    except HTTPError as err:
+                        if err.code not in (403, 405):
+                            raise
+                        # retry with GET if that fails, some servers
+                        # don't like HEAD requests and reply with 403 or 405
+                        req = Request(req_url)
+                        f = opener.open(req, **kwargs)
+                        f.close()
+            except HTTPError as err:
+                if err.code == 401:
+                    # We'll take "Unauthorized" as working.
+                    return 'working', ' - unauthorized', 0
+                else:
+                    return 'broken', str(err), 0
+            except Exception as err:
+                return 'broken', str(err), 0
+            if f.url.rstrip('/') == req_url.rstrip('/'):
+                return 'working', '', 0
+            else:
+                new_url = f.url
+                if anchor:
+                    new_url += '#' + anchor
+                code = getattr(req, 'redirect_code', 0)
+                return 'redirected', new_url, code
+
         def check():
             # check for various conditions without bothering the network
-            if len(uri) == 0 or uri[0] == '#' or \
-               uri[0:7] == 'mailto:' or uri[0:4] == 'ftp:':
+            if len(uri) == 0 or uri.startswith(('#', 'mailto:', 'ftp:')):
                 return 'unchecked', '', 0
-            elif not (uri[0:5] == 'http:' or uri[0:6] == 'https:'):
+            elif not uri.startswith(('http:', 'https:')):
                 return 'local', '', 0
             elif uri in self.good:
                 return 'working', 'old', 0
@@ -154,71 +224,20 @@ class CheckExternalLinksBuilder(Builder):
                 if rex.match(uri):
                     return 'ignored', '', 0
 
-            # split off anchor
-            if '#' in uri:
-                req_url, hash = uri.split('#', 1)
-            else:
-                req_url = uri
-                hash = None
-
-            # handle non-ASCII URIs
-            try:
-                req_url.encode('ascii')
-            except UnicodeError:
-                req_url = encode_uri(req_url)
-
             # need to actually check the URI
-            try:
-                if hash and self.app.config.linkcheck_anchors:
-                    # Read the whole document and see if #hash exists
-                    req = Request(req_url)
-                    f = opener.open(req, **kwargs)
-                    encoding = 'utf-8'
-                    if hasattr(f.headers, 'get_content_charset'):
-                        encoding = f.headers.get_content_charset() or encoding
-                    else:
-                        encoding = get_content_charset(f) or encoding
-                    found = check_anchor(TextIOWrapper(f, encoding), unquote(hash))
-                    f.close()
+            for _ in range(self.app.config.linkcheck_retries):
+                status, info, code = check_uri()
+                if status != "broken":
+                    break
 
-                    if not found:
-                        raise Exception("Anchor '%s' not found" % hash)
-                else:
-                    try:
-                        # try a HEAD request, which should be easier on
-                        # the server and the network
-                        req = HeadRequest(req_url)
-                        f = opener.open(req, **kwargs)
-                        f.close()
-                    except HTTPError as err:
-                        if err.code != 405:
-                            raise
-                        # retry with GET if that fails, some servers
-                        # don't like HEAD requests and reply with 405
-                        req = Request(req_url)
-                        f = opener.open(req, **kwargs)
-                        f.close()
-            except HTTPError as err:
-                if err.code == 401:
-                    # We'll take "Unauthorized" as working.
-                    self.good.add(uri)
-                    return 'working', ' - unauthorized', 0
-                else:
-                    self.broken[uri] = str(err)
-                    return 'broken', str(err), 0
-            except Exception as err:
-                self.broken[uri] = str(err)
-                return 'broken', str(err), 0
-            if f.url.rstrip('/') == req_url.rstrip('/'):
+            if status == "working":
                 self.good.add(uri)
-                return 'working', '', 0
-            else:
-                new_url = f.url
-                if hash:
-                    new_url += '#' + hash
-                code = getattr(req, 'redirect_code', 0)
-                self.redirected[uri] = (new_url, code)
-                return 'redirected', new_url, code
+            elif status == "broken":
+                self.broken[uri] = info
+            elif status == "redirected":
+                self.redirected[uri] = (info, code)
+
+            return (status, info, code)
 
         while True:
             uri, docname, lineno = self.wqueue.get()
