@@ -5,44 +5,51 @@
 
     Utilities parsing and analyzing Python code.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import re
 import sys
 from os import path
-from cStringIO import StringIO
 
+from six import iteritems, text_type, BytesIO, StringIO
+
+from sphinx import package_dir
 from sphinx.errors import PycodeError
 from sphinx.pycode import nodes
 from sphinx.pycode.pgen2 import driver, token, tokenize, parse, literals
-from sphinx.util import get_module_source
+from sphinx.util import get_module_source, detect_encoding
+from sphinx.util.pycompat import TextIOWrapper
 from sphinx.util.docstrings import prepare_docstring, prepare_commentdoc
 
 
 # load the Python grammar
-_grammarfile = path.join(path.dirname(__file__), 'Grammar.txt')
+_grammarfile = path.join(package_dir, 'pycode',
+                         'Grammar-py%d.txt' % sys.version_info[0])
 pygrammar = driver.load_grammar(_grammarfile)
 pydriver = driver.Driver(pygrammar, convert=nodes.convert)
 
+
 # an object with attributes corresponding to token and symbol names
-class sym: pass
-for k, v in pygrammar.symbol2number.iteritems():
+class sym:
+    pass
+
+
+for k, v in iteritems(pygrammar.symbol2number):
     setattr(sym, k, v)
-for k, v in token.tok_name.iteritems():
+for k, v in iteritems(token.tok_name):
     setattr(sym, v, k)
 
 # a dict mapping terminal and nonterminal numbers to their names
 number2name = pygrammar.number2symbol.copy()
 number2name.update(token.tok_name)
 
-
-# a regex to recognize coding cookies
-_coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
-
 _eq = nodes.Leaf(token.EQUAL, '=')
 
+emptyline_re = re.compile('^\s*(#.*)?$')
+
 
 class AttrDocVisitor(nodes.NodeVisitor):
     """
@@ -84,10 +91,30 @@ class AttrDocVisitor(nodes.NodeVisitor):
             self.in_init -= 1
 
     def visit_expr_stmt(self, node):
-        """Visit an assignment which may have a special comment before it."""
+        """Visit an assignment which may have a special comment before (or
+        after) it.
+        """
         if _eq not in node.children:
             # not an assignment (we don't care for augmented assignments)
             return
+        # look *after* the node; there may be a comment prefixing the NEWLINE
+        # of the simple_stmt
+        parent = node.parent
+        idx = parent.children.index(node) + 1
+        while idx < len(parent):
+            if parent[idx].type == sym.SEMI:
+                idx += 1
+                continue  # skip over semicolon
+            if parent[idx].type == sym.NEWLINE:
+                prefix = parent[idx].get_prefix()
+                if not isinstance(prefix, text_type):
+                    prefix = prefix.decode(self.encoding)
+                docstring = prepare_commentdoc(prefix)
+                if docstring:
+                    self.add_docstring(node, docstring)
+                    return  # don't allow docstrings both before and after
+            break
+        # now look *before* the node
         pnode = node[0]
         prefix = pnode.get_prefix()
         # if the assignment is the first statement on a new indentation
@@ -98,7 +125,8 @@ class AttrDocVisitor(nodes.NodeVisitor):
             if not pnode or pnode.type not in (token.INDENT, token.DEDENT):
                 break
             prefix = pnode.get_prefix()
-        prefix = prefix.decode(self.encoding)
+        if not isinstance(prefix, text_type):
+            prefix = prefix.decode(self.encoding)
         docstring = prepare_commentdoc(prefix)
         self.add_docstring(node, docstring)
 
@@ -111,7 +139,7 @@ class AttrDocVisitor(nodes.NodeVisitor):
         if not prev:
             return
         if prev.type == sym.simple_stmt and \
-               prev[0].type == sym.expr_stmt and _eq in prev[0].children:
+           prev[0].type == sym.expr_stmt and _eq in prev[0].children:
             # need to "eval" the string because it's returned in its
             # original form
             docstring = literals.evalString(node[0].value, self.encoding)
@@ -124,16 +152,16 @@ class AttrDocVisitor(nodes.NodeVisitor):
             target = node[i]
             if self.in_init and self.number2name[target.type] == 'power':
                 # maybe an attribute assignment -- check necessary conditions
-                if (# node must have two children
-                    len(target) != 2 or
-                    # first child must be "self"
-                    target[0].type != token.NAME or target[0].value != 'self' or
-                    # second child must be a "trailer" with two children
-                    self.number2name[target[1].type] != 'trailer' or
-                    len(target[1]) != 2 or
-                    # first child must be a dot, second child a name
-                    target[1][0].type != token.DOT or
-                    target[1][1].type != token.NAME):
+                if (  # node must have two children
+                        len(target) != 2 or
+                        # first child must be "self"
+                        target[0].type != token.NAME or target[0].value != 'self' or
+                        # second child must be a "trailer" with two children
+                        self.number2name[target[1].type] != 'trailer' or
+                        len(target[1]) != 2 or
+                        # first child must be a dot, second child a name
+                        target[1][0].type != token.DOT or
+                        target[1][1].type != token.NAME):
                     continue
                 name = target[1][1].value
             elif target.type != token.NAME:
@@ -154,15 +182,17 @@ class ModuleAnalyzer(object):
 
     @classmethod
     def for_string(cls, string, modname, srcname='<string>'):
-        return cls(StringIO(string), modname, srcname)
+        if isinstance(string, bytes):
+            return cls(BytesIO(string), modname, srcname)
+        return cls(StringIO(string), modname, srcname, decoded=True)
 
     @classmethod
     def for_file(cls, filename, modname):
         if ('file', filename) in cls.cache:
             return cls.cache['file', filename]
         try:
-            fileobj = open(filename, 'r')
-        except Exception, err:
+            fileobj = open(filename, 'rb')
+        except Exception as err:
             raise PycodeError('error opening %r' % filename, err)
         obj = cls(fileobj, modname, filename)
         cls.cache['file', filename] = obj
@@ -182,26 +212,32 @@ class ModuleAnalyzer(object):
                 obj = cls.for_string(source, modname)
             else:
                 obj = cls.for_file(source, modname)
-        except PycodeError, err:
+        except PycodeError as err:
             cls.cache['module', modname] = err
             raise
         cls.cache['module', modname] = obj
         return obj
 
-    def __init__(self, source, modname, srcname):
+    def __init__(self, source, modname, srcname, decoded=False):
         # name of the module
         self.modname = modname
         # name of the source file
         self.srcname = srcname
         # file-like object yielding source lines
         self.source = source
-        # will be changed when found by parse()
-        self.encoding = sys.getdefaultencoding()
 
         # cache the source code as well
         pos = self.source.tell()
-        self.code = self.source.read()
-        self.source.seek(pos)
+        if not decoded:
+            self.encoding = detect_encoding(self.source.readline)
+            self.source.seek(pos)
+            self.code = self.source.read().decode(self.encoding)
+            self.source.seek(pos)
+            self.source = TextIOWrapper(self.source, self.encoding)
+        else:
+            self.encoding = None
+            self.code = self.source.read()
+            self.source.seek(pos)
 
         # will be filled by tokenize()
         self.tokens = None
@@ -217,7 +253,10 @@ class ModuleAnalyzer(object):
         """Generate tokens from the source."""
         if self.tokens is not None:
             return
-        self.tokens = list(tokenize.generate_tokens(self.source.readline))
+        try:
+            self.tokens = list(tokenize.generate_tokens(self.source.readline))
+        except tokenize.TokenError as err:
+            raise PycodeError('tokenizing failed', err)
         self.source.close()
 
     def parse(self):
@@ -227,15 +266,8 @@ class ModuleAnalyzer(object):
         self.tokenize()
         try:
             self.parsetree = pydriver.parse_tokens(self.tokens)
-        except parse.ParseError, err:
+        except parse.ParseError as err:
             raise PycodeError('parsing failed', err)
-        # find the source code encoding, if present
-        comments = self.parsetree.get_prefix()
-        for line in comments.splitlines()[:2]:
-            match = _coding_re.search(line)
-            if match is not None:
-                self.encoding = match.group(1)
-                break
 
     def find_attr_docs(self, scope=''):
         """Find class and module-level attributes and their documentation."""
@@ -262,23 +294,25 @@ class ModuleAnalyzer(object):
         indent = 0
         defline = False
         expect_indent = False
-        def tokeniter(ignore = (token.COMMENT, token.NL)):
+        emptylines = 0
+
+        def tokeniter(ignore = (token.COMMENT,)):
             for tokentup in self.tokens:
                 if tokentup[0] not in ignore:
                     yield tokentup
         tokeniter = tokeniter()
         for type, tok, spos, epos, line in tokeniter:
-            if expect_indent:
+            if expect_indent and type != token.NL:
                 if type != token.INDENT:
                     # no suite -- one-line definition
                     assert stack
                     dtype, fullname, startline, _ = stack.pop()
                     endline = epos[0]
                     namespace.pop()
-                    result[fullname] = (dtype, startline, endline)
+                    result[fullname] = (dtype, startline, endline - emptylines)
                 expect_indent = False
             if tok in ('def', 'class'):
-                name = tokeniter.next()[1]
+                name = next(tokeniter)[1]
                 namespace.append(name)
                 fullname = '.'.join(namespace)
                 stack.append((tok, fullname, spos[0], indent))
@@ -294,7 +328,7 @@ class ModuleAnalyzer(object):
                     dtype, fullname, startline, _ = stack.pop()
                     endline = spos[0]
                     namespace.pop()
-                    result[fullname] = (dtype, startline, endline)
+                    result[fullname] = (dtype, startline, endline - emptylines)
             elif type == token.NEWLINE:
                 # if this line contained a definition, expect an INDENT
                 # to start the suite; if there is no such INDENT
@@ -302,24 +336,32 @@ class ModuleAnalyzer(object):
                 if defline:
                     defline = False
                     expect_indent = True
+                emptylines = 0
+            elif type == token.NL:
+                # count up if line is empty or comment only
+                if emptyline_re.match(line):
+                    emptylines += 1
+                else:
+                    emptylines = 0
         self.tags = result
         return result
 
 
 if __name__ == '__main__':
-    import time, pprint
+    import time
+    import pprint
     x0 = time.time()
-    #ma = ModuleAnalyzer.for_file(__file__.rstrip('c'), 'sphinx.builders.html')
+    # ma = ModuleAnalyzer.for_file(__file__.rstrip('c'), 'sphinx.builders.html')
     ma = ModuleAnalyzer.for_file('sphinx/environment.py',
                                  'sphinx.environment')
     ma.tokenize()
     x1 = time.time()
     ma.parse()
     x2 = time.time()
-    #for (ns, name), doc in ma.find_attr_docs().iteritems():
-    #    print '>>', ns, name
-    #    print '\n'.join(doc)
+    # for (ns, name), doc in iteritems(ma.find_attr_docs()):
+    #     print '>>', ns, name
+    #     print '\n'.join(doc)
     pprint.pprint(ma.find_tags())
     x3 = time.time()
-    #print nodes.nice_repr(ma.parsetree, number2name)
-    print "tokenizing %.4f, parsing %.4f, finding %.4f" % (x1-x0, x2-x1, x3-x2)
+    # print nodes.nice_repr(ma.parsetree, number2name)
+    print("tokenizing %.4f, parsing %.4f, finding %.4f" % (x1-x0, x2-x1, x3-x2))
