@@ -17,7 +17,7 @@ from os import path
 
 from sphinx.util import jsdump, rpartition
 from sphinx.util.pycompat import htmlescape
-
+from sphinx.search.jssplitter import splitter_code
 
 class SearchLanguage(object):
     """
@@ -136,6 +136,7 @@ languages = {
     'ru': 'sphinx.search.ru.SearchRussian',
     'sv': 'sphinx.search.sv.SearchSwedish',
     'tr': 'sphinx.search.tr.SearchTurkish',
+    'zh': 'sphinx.search.zh.SearchChinese',
 }
 
 
@@ -240,6 +241,7 @@ class IndexBuilder(object):
                 self.js_scorer_code = fp.read().decode('utf-8')
         else:
             self.js_scorer_code = u''
+        self.js_splitter_code = splitter_code
 
     def load(self, stream, format):
         """Reconstruct from frozen data."""
@@ -276,9 +278,9 @@ class IndexBuilder(object):
         rv = {}
         otypes = self._objtypes
         onames = self._objnames
-        for domainname, domain in iteritems(self.env.domains):
+        for domainname, domain in sorted(iteritems(self.env.domains)):
             for fullname, dispname, type, docname, anchor, prio in \
-                    domain.get_objects():
+                    sorted(domain.get_objects()):
                 # XXX use dispname?
                 if docname not in fn2index:
                     continue
@@ -361,7 +363,7 @@ class IndexBuilder(object):
             try:
                 return self._stem_cache[word]
             except KeyError:
-                self._stem_cache[word] = self.lang.stem(word)
+                self._stem_cache[word] = self.lang.stem(word).lower()
                 return self._stem_cache[word]
         _filter = self.lang.word_filter
 
@@ -380,6 +382,7 @@ class IndexBuilder(object):
             search_language_stemming_code = self.lang.js_stemmer_code,
             search_language_stop_words = jsdump.dumps(sorted(self.lang.stopwords)),
             search_scorer_tool = self.js_scorer_code,
+            search_word_splitter_code = self.js_splitter_code,
         )
 
     def get_js_stemmer_rawcode(self):
