@@ -182,6 +182,37 @@ def copy_static_entry(source, targetdir, builder, context={},
                               exclude_matchers=exclude_matchers)
 
 
+def copy_extra_entry(source, targetdir, exclude_matchers=()):
+    """Copy a HTML builder extra_path entry from source to targetdir.
+
+    Handles all possible cases of files, directories and subdirectories.
+    """
+    def excluded(path):
+        relpath = relative_path(os.path.dirname(source), path)
+        return any(matcher(relpath) for matcher in exclude_matchers)
+
+    def copy_extra_file(source_, targetdir_):
+        if not excluded(source_):
+            target = path.join(targetdir_, os.path.basename(source_))
+            copyfile(source_, target)
+
+    if os.path.isfile(source):
+        copy_extra_file(source, targetdir)
+        return
+
+    for root, dirs, files in os.walk(source):
+        reltargetdir = os.path.join(targetdir, relative_path(source, root))
+        for dir in dirs[:]:
+            if excluded(os.path.join(root, dir)):
+                dirs.remove(dir)
+            else:
+                target = os.path.join(reltargetdir, dir)
+                if not path.exists(target):
+                    os.mkdir(target)
+        for file in files:
+            copy_extra_file(os.path.join(root, file), reltargetdir)
+
+
 _DEBUG_HEADER = '''\
 # Sphinx version: %s
 # Python version: %s (%s)
@@ -484,27 +515,6 @@ class PeekableIterator(object):
         return item
 
 
-def get_figtype(node):
-    """Return figtype for given node."""
-    def has_child(node, cls):
-        return any(isinstance(child, cls) for child in node)
-
-    from docutils import nodes
-    if isinstance(node, nodes.figure):
-        return 'figure'
-    elif isinstance(node, nodes.image) and isinstance(node.parent, nodes.figure):
-        # bare image node is not supported because it doesn't have caption and
-        # no-caption-target isn't a numbered figure.
-        return 'figure'
-    elif isinstance(node, nodes.table):
-        return 'table'
-    elif isinstance(node, nodes.container):
-        if has_child(node, nodes.literal_block):
-            return 'code-block'
-
-    return None
-
-
 def import_object(objname, source=None):
     try:
         module, name = objname.rsplit('.', 1)
@@ -532,3 +542,12 @@ def encode_uri(uri):
                  for (q, v) in parse_qsl(split[3]))
     split[3] = urlencode(query).decode('ascii')
     return urlunsplit(split)
+
+
+def split_docinfo(text):
+    docinfo_re = re.compile('\A((?:\s*:\w+:.*?\n)+)', re.M)
+    result = docinfo_re.split(text, 1)
+    if len(result) == 1:
+        return '', result[0]
+    else:
+        return result[1:]
