@@ -20,6 +20,8 @@ from itertools import product
 
 from six import PY3, text_type, exec_
 
+NoneType = type(None)
+
 # ------------------------------------------------------------------------------
 # Python 2/3 compatibility
 
@@ -78,7 +80,7 @@ else:
     # error handler
     sys_encoding = __import__('locale').getpreferredencoding()
     # use Python 3 name
-    from cgi import escape as htmlescape  # noqa: 2.6, 2.7
+    from cgi import escape as htmlescape  # noqa: F401
 
     class UnicodeMixin(object):
         """Mixin class to handle defining the proper __str__/__unicode__
@@ -145,6 +147,7 @@ class _DeprecationWrapper(object):
             return self._deprecated[attr]
         return getattr(self._mod, attr)
 
+
 sys.modules[__name__] = _DeprecationWrapper(sys.modules[__name__], dict(
     zip_longest = zip_longest,
     product = product,
