@@ -9,6 +9,8 @@
     :license: BSD, see LICENSE for details.
 """
 
+import warnings
+
 from docutils import nodes
 from docutils.writers.manpage import (
     MACRO_DEF,
@@ -18,8 +20,8 @@ from docutils.writers.manpage import (
 
 from sphinx import addnodes
 from sphinx.locale import admonitionlabels, _
-from sphinx.util.osutil import ustrftime
 from sphinx.util.compat import docutils_version
+from sphinx.util.i18n import format_date
 
 
 class ManualPageWriter(Writer):
@@ -95,8 +97,8 @@ class ManualPageTranslator(BaseTranslator):
         if builder.config.today:
             self._docinfo['date'] = builder.config.today
         else:
-            self._docinfo['date'] = ustrftime(builder.config.today_fmt or
-                                              _('%B %d, %Y'))
+            self._docinfo['date'] = format_date(builder.config.today_fmt or _('%b %d, %Y'),
+                                                language=builder.config.language)
         self._docinfo['copyright'] = builder.config.copyright
         self._docinfo['version'] = builder.config.version
         self._docinfo['manual_group'] = builder.config.project
@@ -200,7 +202,16 @@ class ManualPageTranslator(BaseTranslator):
     def depart_versionmodified(self, node):
         self.depart_paragraph(node)
 
+    # overwritten -- don't make whole of term bold if it includes strong node
+    def visit_term(self, node):
+        if node.traverse(nodes.strong):
+            self.body.append('\n')
+        else:
+            BaseTranslator.visit_term(self, node)
+
     def visit_termsep(self, node):
+        warnings.warn('sphinx.addnodes.termsep will be removed at Sphinx-1.5',
+                      DeprecationWarning)
         self.body.append(', ')
         raise nodes.SkipNode
 
@@ -375,6 +386,12 @@ class ManualPageTranslator(BaseTranslator):
     def depart_abbreviation(self, node):
         pass
 
+    def visit_manpage(self, node):
+        return self.visit_strong(node)
+
+    def depart_manpage(self, node):
+        return self.depart_strong(node)
+
     # overwritten: handle section titles better than in 0.6 release
     def visit_title(self, node):
         if isinstance(node.parent, addnodes.seealso):
