@@ -11,10 +11,13 @@
 
 from six import BytesIO
 
+import pickle
+from docutils import nodes
 from textwrap import dedent
 from sphinx.errors import SphinxError
+import sphinx.builders.linkcheck
 
-from util import with_app, rootdir, tempdir, SkipTest, TestApp
+from util import with_app, with_tempdir, rootdir, tempdir, SkipTest, TestApp
 
 try:
     from docutils.writers.manpage import Writer as ManWriter
@@ -29,7 +32,6 @@ class MockOpener(object):
             url = req.url
         return result()
 
-import sphinx.builders.linkcheck
 sphinx.builders.linkcheck.opener = MockOpener()
 
 
@@ -68,22 +70,25 @@ def test_build_all():
 
     # note: no 'html' - if it's ok with dirhtml it's ok with html
     for buildername in ['dirhtml', 'singlehtml', 'latex', 'texinfo', 'pickle',
-                        'json', 'text', 'htmlhelp', 'qthelp', 'epub',
+                        'json', 'text', 'htmlhelp', 'qthelp', 'epub', 'epub3',
                         'applehelp', 'changes', 'xml', 'pseudoxml', 'man',
                         'linkcheck']:
         yield verify_build, buildername, srcdir
 
 
-@with_app(buildername='text')
-def test_master_doc_not_found(app, status, warning):
-    (app.srcdir / 'contents.txt').move(app.srcdir / 'contents.txt.bak')
+@with_tempdir
+def test_master_doc_not_found(tmpdir):
+    (tmpdir / 'conf.py').write_text('master_doc = "index"')
+    assert tmpdir.listdir() == ['conf.py']
+
     try:
+        app = TestApp(buildername='dummy', srcdir=tmpdir)
         app.builder.build_all()
         assert False  # SphinxError not raised
     except Exception as exc:
         assert isinstance(exc, SphinxError)
     finally:
-        (app.srcdir / 'contents.txt.bak').move(app.srcdir / 'contents.txt')
+        app.cleanup()
 
 
 @with_app(buildername='text', testroot='circular')
@@ -108,3 +113,51 @@ def test_numbered_circular_toctree(app, status, warning):
     assert (
         'circular toctree references detected, ignoring: '
         'contents <- sub <- contents') in warnings
+
+
+@with_app(buildername='dummy', testroot='image-glob')
+def test_image_glob(app, status, warning):
+    app.builder.build_all()
+
+    # index.rst
+    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
+
+    assert isinstance(doctree[0][1], nodes.image)
+    assert doctree[0][1]['candidates'] == {'*': 'rimg.png'}
+    assert doctree[0][1]['uri'] == 'rimg.png'
+
+    assert isinstance(doctree[0][2], nodes.figure)
+    assert isinstance(doctree[0][2][0], nodes.image)
+    assert doctree[0][2][0]['candidates'] == {'*': 'rimg.png'}
+    assert doctree[0][2][0]['uri'] == 'rimg.png'
+
+    assert isinstance(doctree[0][3], nodes.image)
+    assert doctree[0][3]['candidates'] == {'application/pdf': 'img.pdf',
+                                           'image/gif': 'img.gif',
+                                           'image/png': 'img.png'}
+    assert doctree[0][3]['uri'] == 'img.*'
+
+    assert isinstance(doctree[0][4], nodes.figure)
+    assert isinstance(doctree[0][4][0], nodes.image)
+    assert doctree[0][4][0]['candidates'] == {'application/pdf': 'img.pdf',
+                                              'image/gif': 'img.gif',
+                                              'image/png': 'img.png'}
+    assert doctree[0][4][0]['uri'] == 'img.*'
+
+    # subdir/index.rst
+    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
+
+    assert isinstance(doctree[0][1], nodes.image)
+    assert doctree[0][1]['candidates'] == {'*': 'subdir/rimg.png'}
+    assert doctree[0][1]['uri'] == 'subdir/rimg.png'
+
+    assert isinstance(doctree[0][2], nodes.image)
+    assert doctree[0][2]['candidates'] == {'application/pdf': 'subdir/svgimg.pdf',
+                                           'image/svg+xml': 'subdir/svgimg.svg'}
+    assert doctree[0][2]['uri'] == 'subdir/svgimg.*'
+
+    assert isinstance(doctree[0][3], nodes.figure)
+    assert isinstance(doctree[0][3][0], nodes.image)
+    assert doctree[0][3][0]['candidates'] == {'application/pdf': 'subdir/svgimg.pdf',
+                                              'image/svg+xml': 'subdir/svgimg.svg'}
+    assert doctree[0][3][0]['uri'] == 'subdir/svgimg.*'
