@@ -9,11 +9,12 @@
     :license: BSD, see LICENSE for details.
 """
 
-import re
-
 from util import with_app
 
 
-@with_app(testroot='ext-ifconfig')
+@with_app(buildername='text', testroot='ext-ifconfig')
 def test_ifconfig(app, status, warning):
     app.builder.build_all()
+    result = (app.outdir / 'index.txt').text()
+    assert 'spam' in result
+    assert 'ham' not in result
