@@ -181,10 +181,11 @@ def test_quickstart_all_answers(tempdir):
         'intersphinx': 'no',
         'todo': 'y',
         'coverage': 'no',
-        'pngmath': 'N',
+        'imgmath': 'N',
         'mathjax': 'no',
         'ifconfig': 'no',
         'viewcode': 'no',
+        'githubpages': 'no',
         'Create Makefile': 'no',
         'Create Windows command file': 'no',
         'Do you want to use the epub builder': 'yes',
