@@ -20,6 +20,7 @@ from util import with_app
 
 settings = parser = None
 
+
 def setup_module():
     global settings, parser
     optparser = frontend.OptionParser(components=(rst.Parser,))
@@ -33,6 +34,7 @@ FILE_CONTENTS = '''\
 test that non-comments are indexed: fermion
 '''
 
+
 def test_wordcollector():
     doc = utils.new_document(b'test data', settings)
     doc['file'] = 'dummy'
@@ -44,7 +46,7 @@ def test_wordcollector():
     assert 'fermion' in ix._mapping
 
 
-@with_app()
+@with_app(testroot='ext-viewcode')
 def test_objects_are_escaped(app, status, warning):
     app.builder.build_all()
     searchindex = (app.outdir / 'searchindex.js').text()
